@@ -1,6 +1,6 @@
 package Treex::Tool::Parser::MSTperl::Config;
-BEGIN {
-  $Treex::Tool::Parser::MSTperl::Config::VERSION = '0.07298';
+{
+  $Treex::Tool::Parser::MSTperl::Config::VERSION = '0.08055';
 }
 
 use Moose;
@@ -28,9 +28,9 @@ has 'SEQUENCE_BOUNDARY_LABEL' => (
 );
 
 has 'VITERBI_STATES_NUM_THRESHOLD' => (
-    is      => 'ro',
+    is      => 'rw',
     isa     => 'Int',
-    default => 50,
+    default => 5,
 );
 
 # stopping criterion of EM algorithm (when the sum of change of smoothing
@@ -66,8 +66,20 @@ has 'labelledFeaturesControl' => (
     is  => 'rw',
 );
 
+has 'imlabelledFeaturesControl' => (
+    isa => 'Maybe[Treex::Tool::Parser::MSTperl::FeaturesControl]',
+    is  => 'rw',
+);
+
 # CONFIGURATION
 
+# only assigning is_member (as opposed to afun labelling)
+has 'is_member_labelling' => (
+    is      => 'ro',
+    isa     => 'Bool',
+    default => '0',
+);
+
 # training mode or parsing mode
 has 'training' => (
     is      => 'ro',
@@ -122,7 +134,7 @@ has 'label' => (
 sub _label_set {
     my ( $self, $label ) = @_;
 
-    # set index of parent's ord field
+    # set index of label field
     my $label_index = $self->field_name2index($label);
     $self->label_field_index($label_index);
 
@@ -136,6 +148,30 @@ has 'label_field_index' => (
     #    default => 'undef',
 );
 
+has 'ismember' => (
+    is      => 'rw',
+    isa     => 'Str',
+    trigger => \&_ismember_set,
+);
+
+# sets ismember_field_index
+sub _ismember_set {
+    my ( $self, $ismember ) = @_;
+
+    # set index of ismember field
+    my $ismember_index = $self->field_name2index($ismember);
+    $self->ismember_field_index($ismember_index);
+
+    return;
+}
+
+has 'ismember_field_index' => (
+    is  => 'rw',
+    isa => 'Maybe[Int]',
+
+    #    default => 'undef',
+);
+
 has 'root_field_values' => (
     is      => 'rw',
     isa     => 'ArrayRef[Str]',
@@ -171,6 +207,12 @@ has 'labeller_number_of_iterations' => (
     default => 10,
 );
 
+has 'imlabeller_number_of_iterations' => (
+    isa     => 'Int',
+    is      => 'rw',
+    default => 10,
+);
+
 has 'use_edge_features_cache' => (
     is      => 'rw',
     isa     => 'Bool',
@@ -183,6 +225,12 @@ has 'labeller_use_edge_features_cache' => (
     default => '0',
 );
 
+has 'imlabeller_use_edge_features_cache' => (
+    is      => 'rw',
+    isa     => 'Bool',
+    default => '0',
+);
+
 # using cache turned off to fit into RAM by default
 # turn on if training with a lot of RAM or on small training data
 # turned off when parsing (does not make any sense for parsing)
@@ -363,9 +411,12 @@ sub BUILD {
             'label',
             'use_edge_features_cache',
             'labeller_use_edge_features_cache',
+            'imlabeller_use_edge_features_cache',
             'number_of_iterations',
             'labeller_number_of_iterations',
+            'imlabeller_number_of_iterations',
             'labeller_algorithm',
+            'VITERBI_STATES_NUM_THRESHOLD',
         );
 
         # name => required?
@@ -424,8 +475,25 @@ sub BUILD {
             );
         }
 
+        # imlabeller features
+        if ($config->[0]->{imlabeller_features}
+            && @{ $config->[0]->{imlabeller_features} }
+            )
+        {
+            $self->imlabelledFeaturesControl(
+                Treex::Tool::Parser::MSTperl::FeaturesControl->new(
+                    'config' => $self,
+                    'feature_codes_from_config'
+                        => $config->[0]->{imlabeller_features},
+                    'use_edge_features_cache'
+                        => $self->imlabeller_use_edge_features_cache,
+                    )
+            );
+        }
+
         if (!$self->unlabelledFeaturesControl
             && !$self->labelledFeaturesControl
+            && !$self->imlabelledFeaturesControl
             )
         {
             croak "MSTperl config file error: No features set!";
@@ -456,7 +524,7 @@ sub field_name2index {
 
             # everything OK -> return the field name
             return $self->field_indexes->{$field_name};
-        } elsif ( $field_name =~ /^[0-9]+$/ ) {
+        } elsif ( $field_name =~ /^-?[0-9]+$/ ) {
 
             # not an actual field name but an integer argument -> keep it
             return $field_name;
@@ -490,7 +558,7 @@ Treex::Tool::Parser::MSTperl::Config
 
 =head1 VERSION
 
-version 0.07298
+version 0.08055
 
 =head1 DESCRIPTION
 
@@ -498,7 +566,9 @@ Handles the configuration of the parser.
 
 =head1 FIELDS
 
-=head2 Fields
+=head2 Data fields
+
+Fields describing fields used with nodes, such as form, pos, lemma...
 
 =over 4
 
@@ -519,10 +589,16 @@ index)
 
 =head2 Settings
 
-The the config file (usually C<config.txt>) is in YAML format.
+Most of the settings are set by a config file in YAML format.
+However, you do not have to understand YAML to be able to change the
+settings provided that you keep things like formating of the file unchanged
+(some whitespaces are significant etc.). Actually only a subset of all
+all that YAML provides is used.
 
-Lines beginning with # are comments and are ignored. Lines that contain
-only whitespace chars or are empty are ignored as well.
+Contents of a line from the # character till the end of the line are comments
+and are ignored (if you need to actually use the # sign, you can quote it -
+eg. C<'#empty#'> is interpreted as C<#empty#>). Lines that contain only
+whitespace chars or are empty are ignored as well.
 
 Some of the settings are ignored when in parsing mode (i.e. not training).
 These are use_edge_features_cache (turned off) and number_of_iterations
@@ -577,6 +653,105 @@ See L<Treex::Tool::Parser::MSTperl::FeaturesControl> for details.
 
 =back
 
+=head3 Internal technical settings
+
+These fields cannot be set by the config file, their default values are
+hard-coded at beginning of the source code and they can be set
+on creating the Config object, eg.:
+
+ my $config = Treex::Tool::Parser::MSTperl::Config->new(
+     config_file => 'file.config',
+     VITERBI_STATES_NUM_THRESHOLD => '5',
+     EM_EPSILON => '0.00000001',
+ )
+
+If there is a need, they might be changed to fully config-file-configurable
+settings in future.
+
+=over 4
+
+=item labeller_algorithm
+
+Algorithm used for Viterbi labelling as well as for training. Several
+possibilities are being tried out; if one of them finally significantly
+outscores the other variants, this will become obsolete and get deleted.
+
+=item DEBUG
+
+An integer specifying how much debug information you will be getting while
+running the program, ranging from 0 (no debug info)
+through 1 (progress messages - this is the default setting)
+through 2, 3 and 4 to 5 (more and more debug info).
+
+If you set this value to something higher than 1, you should always redirect
+the output to a file as printing it to the console is very very slow
+(and there is so much info that you wouldn't be able to
+read anything anyway).
+
+This the only read-write field in this section, it is therefore possible
+to change its value not only on creating a new Config instance but also
+while running the program (eg, if you only want to debug only a particular
+part).
+
+=item SEQUENCE_BOUNDARY_LABEL
+
+This is only a technical thing; a label must be assigned to the (basically
+virtual) boundary of a sequence, different from any label used in the data.
+The default value is '###', so if you use this exact label as a valid label in
+your data, change the setting to something else. If nothing goes wrong, you
+should never see this label in the output; however, it is contained in the
+model and used for "transition scores" to score the "transition" between the
+sequence boundary and the first/last node (i.e. it determines the scores of
+labels used as the first or last label in the sequence where no actual
+transition takes place and the transition scores would otherwise get ignored).
+
+=item VITERBI_STATES_NUM_THRESHOLD
+
+Number of states to keep when pruning. The pruning takes place after each
+Viterbi step (i.e. after each computation of possible labels and their scores
+for one edge). For more details see the C<prune> subroutine.
+
+=item EM_EPSILON
+
+Stopping criterion of EM algorithm which is used to compute smoothing
+parameters for linear combination smoothing of transition probabilities
+in some variants of the Labeller.
+(when the sum of change of smoothing
+parameters is lower than the epsilon, the algorithm stops).
+
+=item EM_heldout_data_at
+
+A number between 0 and 1 specifying
+where in training data do heldout data for EM algorithm start
+(eg. 0.75 means that first 75% of sentences
+are training data and the last 25% are heldout data).
+
+The training/heldout data division only affects computation of transition
+probabilities by MLE, it does not affect MIRA training or MLE for emission
+probabilities.
+
+If EM is not used for smoothing, all data are used as training data.
+
+=back
+
+=head2 Technical fields
+
+Provide access to things needed in more than one of the other packages.
+
+=over 4
+
+=item unlabelledFeaturesControl
+
+Provides access to unlabelled features, especially enabling their computation.
+Intance of L<Treex::Tool::Parser::MSTperl::FeaturesControl>.
+
+=item labelledFeaturesControl
+
+Provides access to labeller features, especially enabling their computation.
+Intance of L<Treex::Tool::Parser::MSTperl::FeaturesControl>.
+
+=back
+
 =head1 METHODS
 
 =head2 Settings
@@ -613,4 +788,4 @@ Copyright © 2011 by Institute of Formal and Applied Linguistics,
 Charles University in Prague
 
 This module is free software;
-you can redistribute it and/or modify it under the same terms as Perl itself.
\ No newline at end of file
+you can redistribute it and/or modify it under the same terms as Perl itself.
